@@ -40,15 +40,14 @@ rmd_assess_resid2 <- function(year, obs, fit, fig.cap, label = fig.cap) {
     "```\n")
 }
 
-
-
-rmd_fit_comps <- function(year, obs, fit, type = c("bubble_data", "annual", "bubble_residuals", "mean"), ages = "NULL", CAL_bins = "NULL", fig.cap,
-                          bubble_adj = "10") {
+rmd_fit_comps <- function(year, obs, fit, type = c("bubble_data", "annual", "bubble_residuals", "mean", "heat_residuals"), 
+                          ages = "NULL", CAL_bins = "NULL", N = "NULL", fig.cap,
+                          bubble_adj = "1.5") {
   type <- match.arg(type)
   arg <- paste0("\"", type, "\", CAL_bins = ", CAL_bins, ", ages = ", ages)
   c(paste0("```{r, fig.cap = \"", fig.cap, "\"}"),
     paste0("ind_valid <- rowSums(", obs, ", na.rm = TRUE) > 0"),
-    paste0("if (any(ind_valid)) plot_composition(", year, "[ind_valid], ", obs, "[ind_valid, , drop = FALSE], ", fit, "[ind_valid, , drop = FALSE], plot_type = ", arg, ", bubble_adj = ", bubble_adj, ")"),
+    paste0("if (any(ind_valid)) plot_composition(", year, "[ind_valid], ", obs, "[ind_valid, , drop = FALSE], ", fit, "[ind_valid, , drop = FALSE], plot_type = ", arg, ", N = ", N, "[ind_valid], bubble_adj = ", bubble_adj, ")"),
     "```\n")
 }
 
@@ -103,17 +102,11 @@ rmd_RCM_Find <- function(fig.cap = "Apical F from RCM model. These values may be
     "```\n")
 }
 
-rmd_RCM_sel <- function(fig.cap = "Operating model selectivity among simulations.") {
+rmd_RCM_sel <- function(fig.cap = "Operating model selectivity in the last historical year (all simulations).") {
   c(paste0("```{r, fig.cap = \"", fig.cap, "\"}"),
     "vul <- sapply(report_list, getElement, \"vul_len\")",
-    "if (nsel_block == 1 && all(!is.na(vul))) {",
-    "  matplot(RCMdata@Misc$lbinmid, vul, type = \"l\", col = \"black\",",
-    "          xlab = \"Length\", ylab = \"Selectivity (last historical year)\", ylim = c(0, 1.1))",
-    "} else {",
-    "  if (nsim == 1) V_plot <- matrix(OM@cpars$V[, , nyears], 1, byrow = TRUE) else V_plot <- OM@cpars$V[, , nyears]",
-    "  matplot(age, t(V_plot), type = \"l\", col = \"black\",",
-    "          xlab = \"Age\", ylab = \"Selectivity (last historical year)\", ylim = c(0, 1.1))",
-    "}",
+    "if (nsim == 1) V_plot <- matrix(OM@cpars$V[, , nyears], 1, byrow = TRUE) else V_plot <- OM@cpars$V[, , nyears]",
+    "matplot(age, t(V_plot), type = \"l\", col = \"black\", xlab = \"Age\", ylab = \"Selectivity\", ylim = c(0, 1.1))",
     "abline(h = 0, col = \"grey\")",
     "```\n")
 }
@@ -121,18 +114,18 @@ rmd_RCM_sel <- function(fig.cap = "Operating model selectivity among simulations
 rmd_RCM_fleet_output <- function(ff, f_name) {
   if (ff == 1) header <- "## RCM output {.tabset}\n" else header <- NULL
   ans <- c(paste("### ", f_name[ff], "\n"),
-           paste0("```{r, fig.cap = \"Selectivity of ", f_name[ff], ".\"}"),
+           paste0("```{r, fig.cap = \"Length selectivity of ", f_name[ff], ".\"}"),
            paste0("bl <- unique(RCMdata@sel_block[, ", ff, "])"),
            "vul_bb <- list()",
            "bl_col <- gplots::rich.colors(length(bl))",
            "Year_legend <- character(length(bl))",
            "for(bb in 1:length(bl)) {",
-           "  vul_bb[[bb]] <- sapply(report_list, function(x) x$vul_len[, bl[bb]])",
+           "  vul_bb[[bb]] <- sapply(report_list, function(x) if (!is.null(x$vul_len)) x$vul_len[, bl[bb]] else NA)",
            paste0("  Year_legend[bb] <- Year[RCMdata@sel_block[, ", ff, "] == bl[bb]] %>% range() %>% paste(collapse = \"-\")"),
            "}",
            "test <- vapply(vul_bb, function(x) all(!is.na(x)), logical(1))",
            "if (all(test)) {",
-           paste0("  matplot(RCMdata@Misc$lbinmid, RCMdata@Misc$lbinmid, type = \"n\", xlab = \"Length\", ylim = c(0, 1), ylab = \"Selectivity of Fleet ", ff, "\")"),
+           paste0("  matplot(RCMdata@Misc$lbinmid, RCMdata@Misc$lbinmid, type = \"n\", xlab = \"Length\", ylim = c(0, 1), ylab = \"Selectivity of ", f_name[ff], "\")"),
            "  abline(h = 0, col = \"grey\")",
            "  for(bb in 1:length(bl)) {",
            "    matlines(RCMdata@Misc$lbinmid, vul_bb[[bb]], type = \"l\", col = bl_col[bb], lty = scenario$lty, lwd = scenario$lwd)",
@@ -142,8 +135,8 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "}",
            "```\n",
            "",
-           paste0("```{r, fig.cap = \"Corresponding age-based selectivity of ", f_name[ff], ".\"}"),
-           paste0("matplot(age, age, type = \"n\", xlab = \"Age\", ylim = c(0, 1), ylab = \"Selectivity of Fleet ", ff, "\")"),
+           paste0("```{r, fig.cap = \"Age-based selectivity of ", f_name[ff], ".\"}"),
+           paste0("matplot(age, age, type = \"n\", xlab = \"Age\", ylim = c(0, 1), ylab = \"Selectivity of ", f_name[ff], "\")"),
            "abline(h = 0, col = \"grey\")",
            "",
            "for(bb in 1:length(bl)) {",
@@ -162,7 +155,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "```\n",
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) catch from ", f_name[ff], ".\"}"),
-           paste0("if (any(RCMdata@Chist[, ", ff, "] > 0)) {"),
+           paste0("if (any(RCMdata@Chist > 0, na.rm = TRUE)) {"),
            paste0("  Cpred <- sapply(report_list, function(x) x$Cpred[, ", ff, "])"),
            paste0("  Chist <- RCMdata@Chist[, ", ff, "]"),
            "  ylim <- c(0.9, 1.1) * range(c(Cpred, Chist), na.rm = TRUE)",
@@ -188,7 +181,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            "```\n",
            "",
            paste0("```{r, fig.cap = \"Observed (black) and predicted (red) mean lengths from ", f_name[ff], ".\"}"),
-           paste0("MLpred <- sapply(report_list, function(x) x$MLpred[, ", ff, "])"),
+           paste0("MLpred <- sapply(report_list, function(x) if(!is.null(x$MLpred)) x$MLpred[, ", ff, "] else NA)"),
            paste0("if (any(RCMdata@CAL[, , ", ff, "] > 0, na.rm = TRUE)) {"),
            paste0("  MLobs <- (RCMdata@CAL[, , ", ff, "] %*% RCMdata@Misc$lbinmid)/rowSums(RCMdata@CAL[, , ", ff, "], na.rm = TRUE)"),
            paste0("} else if (RCMdata@MS_type == \"length\" && any(RCMdata@MS[, ", ff, "] > 0, na.rm = TRUE)) MLobs <- RCMdata@MS[, ", ff, "] else MLobs <- NA"),
@@ -219,7 +212,7 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
            paste0("plot_composition_RCM(Year, CAA_plot, RCMdata@CAA[, , ", ff, "], ages = age, dat_col = scenario$col)"),
            "}",
            "```\n",
-           paste0("```{r, fig.cap = \"Predicted age composition from fleet ", ff, ".\"}"),
+           paste0("```{r, fig.cap = \"Predicted age composition from ", f_name[ff], ".\"}"),
            paste0("if (any(RCMdata@CAA[, , ", ff, "] > 0, na.rm = TRUE)) {"),
            paste0("plot_composition_RCM(Year, CAA_plot, ages = age, dat_col = scenario$col)"),
            "}",
@@ -244,7 +237,16 @@ rmd_RCM_fleet_output <- function(ff, f_name) {
 rmd_RCM_index_output <- function(sur, s_name) {
   ans <- c(paste0("### ", s_name[sur], " \n"),
            "",
-           paste0("```{r, fig.cap = \"Selectivity of ", s_name[sur], " in last historical year.\"}"),
+           paste0("```{r, fig.cap = \"Length selectivity of ", s_name[sur], ".\"}"),
+           "ivul_len <- sapply(report_list, function(x) if (!is.null(x$ivul_len)) x$ivul_len[, ", sur, "] else NA)",
+           "if (all(!is.na(ivul_len))) {",
+           paste0("  matplot(RCMdata@Misc$lbinmid, ivul_len, type = \"n\", xlab = \"Length\", ylim = c(0, 1), ylab = \"Selectivity of ", s_name[sur], "\")"),
+           "  abline(h = 0, col = \"grey\")",
+           "  if (!is.null(scenario$names)) legend(\"topleft\", scenario$names, col = scenario$col2, lty = scenario$lty, lwd = scenario$lwd)",
+           "}",
+           "```\n",
+           "",
+           paste0("```{r, fig.cap = \"Age-based selectivity of ", s_name[sur], " in last historical year.\"}"),
            "if (!is.null(report_list[[1]]$ivul)) {",
            paste0("ivul_ff_age <- sapply(report_list, function(x) x$ivul[nyears, , ", sur, "])"),
            paste0("matplot(age, ivul_ff_age, type = \"l\", col = scenario$col2, xlab = \"Age\", ylim = c(0, 1), ylab = \"Selectivity of ", s_name[sur], "\")"),
@@ -528,6 +530,7 @@ rmd_RCM_Hist_compare <- function() {
     "",
     "```{r, fig.cap = \"Comparison of total removals between the OM and RCM.\"}",
     "RCM_C <- sapply(x@Misc, function(xx) rowSums(xx$Cpred)) %>% t()",
+    "if (nrow(RCM_C) == 1) RCM_C <- replicate(x@OM@nsim, as.numeric(RCM_C)) %>% t()",
     "matplot(Year, t(Hist_C), typ = \"o\", col = \"red\", pch = 16, xlab = \"Year\", ylab = \"Total removals\",",
     "        ylim = c(0, 1.1 * max(Hist_C, RCMdata@Chist)))",
     "matlines(Year, t(RCM_C[sims, , drop = FALSE]), col = \"black\")",
@@ -545,7 +548,9 @@ rmd_RCM_Hist_compare <- function() {
 }
 
 plot_composition_RCM <- function(Year, fit, dat = NULL, CAL_bins = NULL, ages = NULL, annual_ylab = "Frequency",
-                                 annual_yscale = c("proportions", "raw"), N = if (is.null(dat)) NULL else round(rowSums(dat)), dat_linewidth = 2, dat_color = "black") {
+                                 annual_yscale = c("proportions", "raw"), 
+                                 N = if (is.null(dat)) NULL else round(rowSums(dat)), 
+                                 dat_linewidth = 2, dat_color = "black") {
   old_par <- par(no.readonly = TRUE)
   on.exit(par(old_par))
   par(mfcol = c(4, 4), mar = rep(0, 4), oma = c(5.1, 5.1, 2.1, 2.1))
